#!/bin/bash
# A common startup script for docker container and standalone
DEFAULT_JAVA_OPTIONS="-Duser.timezone=UTC -Dspring.profiles.active=production"

SERVICE_NAME=distribution
SCRIPT_NAME=${SERVICE_NAME}.sh
DEFAULT_PORT=8080
## By default mac doesnt have pidof command, this little hack should solve this design issue
macPidof() {
    export myPID=$(ps axc|awk "{if (\$5~\"$1\") print \$1}")
    return $myPID
}

exitError() {
    echo -e "\n\033[31mERROR: $1\033[0m\n"

    [ -z "${2}" ] || usage
    exit 1
}

usage() {
    cat << END_USAGE

${SCRIPT_NAME} - script for controlling ${SERVICE_NAME}

Usage: ./${SCRIPT_NAME} <action>

action:    help|start|stop|restart|status


END_USAGE

    exit 1
}

# Process command line options. See usage above for supported options
processOptions () {
    # Get action - help|start|stop|restart|status
    ACTION=${1}
    shift

    if [[ ! "${ACTION}" =~ ^(help|start|stop|restart|status)$ ]]; then
        usage
    fi

    if [[ ! "${ACTION}" =~ ^(help|start|stop|restart|status)$ ]]; then
        usage
    fi

    if [[ "${ACTION}" =~ ^help$ ]]; then
        usage
    fi

    # Process the options
    while [[ $# > 0 ]]; do
        case "$1" in
            *)
                usage
            ;;
        esac
    done

}

addJavaArgs () {
    local extra_java_options=
    local java_options=
    local port=

    getSystemValue "${SERVICE_NAME}.extraJavaOpts" "NOT_SET"
    extra_java_options="${YAML_VALUE}"
    if [[ "${extra_java_options}" == "NOT_SET" ]]; then
        extra_java_options=""
    fi

    getSystemValue "${SERVICE_NAME}.port" "${DEFAULT_PORT}"
    portOption="-Dserver.port=${YAML_VALUE}"

    removeDuplicateJavaOpts "${DEFAULT_JAVA_OPTIONS}" "${extra_java_options}"
    DEFAULT_JAVA_OPTIONS="${UNIQUE_DEFAULT_JAVA_OPTIONS}"

    DISTRIBUTION_JAVA_OPTIONS="${DEFAULT_JAVA_OPTIONS} ${portOption} ${extra_java_options}"
    logger "Final command: ${DISTRIBUTION_JAVA_OPTIONS}"

}

startupActions(){
    testVarPermissions
    syncEtc
    initJava
    checkJavaVersion
    addJavaArgs
    setContextUrl
    exportEnv "shared"
    exportEnv "${SERVICE_NAME}"
    addCertsToJavaKeystore
    displayEnv
    validateSystemYaml
    validateJfrogUrl "${JF_PRODUCT_HOME}"
    validateDbconnection "${JF_PRODUCT_HOME}"
}

sourceScript(){
    local file=$1

    [ ! -z "${file}" ] || errorExit "target file is not passed to source a file"
    [   -f "${file}" ] || errorExit "${file} file is not found"
    source "${file}"   || errorExit "Unable to source ${file}, please check if the $USER user has permissions to perform this action"    
}

initHelpers(){
    local systemYamlHelper="${DISTRIBUTION_BIN_FOLDER}"/systemYamlHelper.sh
    local installerCommon="${DISTRIBUTION_BIN_FOLDER}"/installerCommon.sh
    local distributionCommon="${DISTRIBUTION_BIN_FOLDER}"/distributionCommon.sh

    export YQ_PATH="${DISTRIBUTION_BIN_FOLDER}/../third-party/yq"
    sourceScript "${systemYamlHelper}"
    sourceScript "${distributionCommon}"
    sourceScript "${installerCommon}"
    PRODUCT_NAME=$DISTRIBUTION_LABEL
    # init at each service startup META
    export JF_SYSTEM_YAML="${JF_PRODUCT_HOME}/var/etc/system.yaml"
}

setExecutable() {
    local os=$(uname -s)
    local arch=$(uname -m)

    # Resolve OS
    case ${os} in
        Linux)
            os=linux
        ;;
        Darwin)
            os=darwin
        ;;
        *)
            exitError "OS type ${os} is not supported"
        ;;
    esac

    # Resolve cpu Architecture
    case ${arch} in
        x86_64)
            arch=amd64
        ;;
        *)
            exitError "CPU architecture ${arch} is not supported"
        ;;
    esac
}

isRunning() {
    local ps=
    # Set binary to run
    setExecutable
    local bin=${SERVICE_JAR}
    # Check running state by PID
    if [ -e "${SERVICE_PID}" ]; then
        PID_VALUE=$(cat ${SERVICE_PID})
        if [ -n "${PID_VALUE}" ]; then
            ps=$(ps -ef | grep -m1 "${bin}" | grep -v grep 2> /dev/null | awk '{print $1}')
            if [[ "$(uname -s )" == "Darwin" ]]; then
               macBIN=$(echo $bin| awk -F/ '{print $NF}')
               macPidof "${macBIN}"
               if [[ ! -z "${myPID}" ]];then 
                  export ps="${myPID}"
               fi
            else
               export ps
            fi            
        fi
    else
        # Try and find by process
        ps=$(ps -ef | grep -i ${bin} | grep -v grep 2> /dev/null)
        PID_VALUE=$(echo -n ${ps} | awk '{print $2}')
    fi
    if [ -z "${ps}" ]; then
        if [ -z "${PID_VALUE}" ]; then
            return 1 # Not running
        else
            return 2 # PID exists but process dead
        fi
    else
        return 0 # Is running
    fi
}

runAppCommand() {
   exec $_java ${DISTRIBUTION_JAVA_OPTIONS} -jar "${SERVICE_JAR}"   
}

# Start process in background
start() {
    echo "Starting ${SERVICE_NAME}..."

    # Check if running
    isRunning
    case $? in
        0)
            echo "${SERVICE_NAME} is already running (PID: ${PID_VALUE})"
            restart
            return
        ;;
        1)
#            echo "${SERVICE_NAME} not running. Proceed to start it up."
        ;;
        2)
            echo "PID exists (${PID_VALUE}), but ${SERVICE_NAME} not running. Proceed to start it up."
        ;;
        *)
            exitError "Function isRunning() returned an unknown value ($?)"
        ;;
    esac

    if [ -n "$SERVICE_PID" ];
    then
        mkdir -p $(dirname "$SERVICE_PID") || \
        errorExit "Could not create dir for $SERVICE_PID";
    fi

    # Run from  JF_PRODUCT_HOME/var to avoid nohup errors
    cd ${JF_PRODUCT_HOME}/var

    startupActions
    if $(isConsoleLogDisabled >/dev/null 2>&1); then
        runAppCommand >/dev/null 2>&1 &
    else
        _createConsoleLog
        runAppCommand >>"${JF_PRODUCT_HOME}"/var/log/console.log 2>&1 &
    fi

    local pid=$!
    echo -n "${pid}" > "${SERVICE_PID}"

    echo "${SERVICE_NAME} started. PID: ${pid}"
}

# Start process in foreground
run(){
    startupActions
    if $(isConsoleLogDisabled >/dev/null 2>&1); then
        runAppCommand
    else
        runAppCommand > >(tee >( redirectServiceLogsToFile ) ) 2>&1
    fi
}

stop() {
    echo "Stopping ${SERVICE_NAME}..."

    # Check if running
    isRunning
    case $? in
        0)
            echo "${SERVICE_NAME} is running (PID: ${PID_VALUE}). Stopping it..."
        ;;
        1)
            echo "${SERVICE_NAME} not running."
            exit 0
        ;;
        2)
            echo "PID exists (${PID_VALUE}), but ${SERVICE_NAME} not running. Removing PID file..."
            rm -f ${SERVICE_PID}
            exit 0
        ;;
        *)
            exitError "Function isRunning() returned an unknown value ($?)"
        ;;
    esac

    # Kill using PID
    kill ${PID_VALUE} || exitError "Stopping ${SERVICE_NAME} failed"
    rm -f ${SERVICE_PID}
    echo "${SERVICE_NAME} stopped"
}

restart() {
    echo "Restarting ${SERVICE_NAME}..."
    stop
    start
}

status() {
    echo -n "${SERVICE_NAME} is "

    # Check if running
    isRunning
    case $? in
        0)
            echo "running (PID: ${PID_VALUE})"
        ;;
        1)
            echo "not running"
        ;;
        2)
            echo "PID exists (${PID_VALUE}), but ${SERVICE_NAME} not running."
        ;;
        *)
            exitError "Function isRunning() returned an unknown value ($?)"
        ;;
    esac
}

DISTRIBUTION_BIN_FOLDER="$(cd $(dirname "${BASH_SOURCE[0]}")/../../bin && pwd)"
defaultEnv="${DISTRIBUTION_BIN_FOLDER}/distribution.default"

. ${defaultEnv} || exitError "ERROR: $defaultEnv does not exist or not executable"

SERVICE_PID=${JF_PRODUCT_HOME}/app/run/${SERVICE_NAME}.pid
SERVICE_JAR=${JF_PRODUCT_HOME}/app/${SERVICE_NAME}/lib/${SERVICE_NAME}.jar

initHelpers

# run application in the foreground if nothing is passed - docker flow
if [[ $# == 0 ]]; then 
    run
else
    processOptions $*
fi

eval ${ACTION}
